package com.sea.gbgs5;

import com.cinterion.io.ATCommandFailedException;
import java.io.*;
import com.sea.log.Logger;
import com.sea.modem.*;
import com.sea.utils.*;
import java.util.Vector;

public class Control extends Thread {

// --------------------------------------------------
// Public Properties
// --------------------------------------------------
public static LanguageAbstract language = null;
// --------------------------------------------------
// Private Constants
// --------------------------------------------------
// Machine states
private static final int CTRL_INIT = 0;
private static final int CTRL_CREATE = 10;
private static final int CTRL_START = 20;
private static final int CTRL_READY = 1000;
private static final int CTRL_STOP = 2000;
private static final int CTRL_FINISHED = 9999;

// How long sleep when ready [ms]
private static final int READY_DELAY = 20000;

private static final String FILENAME_SERIAL_NUMBER = "file:///A:/sn.txt";

// --------------------------------------------------
// Private Properties
// --------------------------------------------------
private static Control instance = null;
private static boolean bRunning = false;

// Debug Level
public static final int iLogLevel = Logger.INFO;

// Machine state
private static int iMachine;
private static Vector vDeviceInfo;

// Controlled threads
private static Console consl;
private static ConsoleReporter conslrep;
private static Slave slave;
//private static TcpReporter tcprep;
private static Watchdog wdg;

// Controlled AT channels
private static SyncAtCommand ata, atb, atc;

// --------------------------------------------------
// Class Code
// --------------------------------------------------
/**
 * Constructor
 */
public Control() {
  ata = atb = atc = null;
}

/**
 * getInstance()
 */
public static Control getInstance() {
  if (instance == null) {
    instance = new Control();
  }
  return instance;
}

/**
 * startRunning
 */
public void startRunning() {
  bRunning = true;
  this.start();
}

/**
 * stopRunning
 */
public void stopRunning() {
  machineJumpTo(CTRL_STOP);
  instance.interrupt();
}

/**
 * Debug printing
 */
protected static void log(String line) {
  log(line, Logger.INFO);
}

/**
 * Debug printing
 */
protected static void log(String line, int level) {
  if (iLogLevel >= level) {
    Logger.log("@Control", line);
  }
}

/**
 * run
 */
public void run() {

  machineJumpTo(CTRL_INIT);
  while (bRunning) {
    try {
      switch (iMachine) {
        case CTRL_INIT:
          ctrlMachineInit();
          break;
        case CTRL_CREATE:
          ctrlMachineCreate();
          break;
        case CTRL_START:
          ctrlMachineStart();
          break;
        case CTRL_READY:
          ctrlMachineReady();
          break;
        case CTRL_STOP:
          ctrlMachineStop();
          break;
        case CTRL_FINISHED:
          ctrlMachineFinished();
          break;
        default:

          // inpossible state
          machineJumpTo(CTRL_INIT);
      }
    } catch (Exception e) {
      // An exception occured. Go the interrupted step again.
      log("[EXCEPTION] - " + e.toString(), Logger.ERROR);
      e.printStackTrace();
    }
  }

  log("Thread finished.", Logger.ALWAYS);
}

/**
 * ctrlMachineInit
 */
private void ctrlMachineInit() {

  detectSerialNumber();
  vDeviceInfo = prepareDeviceInfo();
  printDeviceInfo(vDeviceInfo);

  log("Comm ports available='" + System.getProperty("microedition.commports") + "'");
  log("Thread started.", Logger.ALWAYS);

  machineJumpTo(CTRL_CREATE);
}

/**
 * ctrlMachineStart
 */
private void ctrlMachineCreate() throws ATCommandFailedException, IOException {

  // Create AT channels
  //TODO Exceptions from AtCommand constructor have to cause RESET of the device
  if (ata == null) {
    ata = new SyncAtCommand(false);
  }
  if (atb == null) {
    atb = new SyncAtCommand(true);
  }
  if (atc == null) {
    atc = new SyncAtCommand(false);
  }

  // Create configuration
  Cfg.init();
  log("Creating Language.");
  language = new LanguageEnglish();  // ``` Udelat zavisle na konfiguraci

  log("Creating Info");
  Info.getInstance();
  log("Creating Console");
  consl = Console.getInstance();
  consl.registerAtCommand(atb);
  consl.setDeviceInfo(vDeviceInfo);
  conslrep = ConsoleReporter.getInstance();
  // Create of thread Modem (creating Modem have to be before next creating Configuration)
  selectStdout("USB");

  // I/O Managers
  log("Creating Watchdog.");
  wdg = Watchdog.getInstance();
  wdg.registerAtCommand(ata);
  log("Creating Slave");
  slave = Slave.getInstance();
  //log("Creating TcpReporter");
  //tcprep = TcpReporter.getInstance();

  machineJumpTo(CTRL_START);
}

/**
 * ctrlMachineStart
 */
private void ctrlMachineStart() {
  log("Starting threads.");

  log("Starting Info");
  Info.getInstance().startRunning();
  consl.startRunning();
  conslrep.startRunning();

  slave.startRunning();
  wdg.startRunning();

  machineJumpTo(CTRL_READY);
}

/**
 * ctrlMachineReady
 */
private void ctrlMachineReady() {

  log(getStatus(0), Logger.ALWAYS);
  log(getStatus(1), Logger.ALWAYS);

  // Watch all other threads and restart if any of them stops working
  if (!Info.getInstance().isAlive() || !wdg.isAlive() || !slave.isAlive()) {
    Watchdog.deviceReset();
  }

  delay(READY_DELAY);
}

/**
 * ctrlMachineStop
 */
private void ctrlMachineStop() {

  log("Stopping threads.");

  // Stop of thread Slave
  try {
    slave.stopRunning();
  } catch (Exception ignored) {
  }

  // Stop of thread Watchdog
  try {
    wdg.stopRunning();
  } catch (Exception ignored) {
  }

  // Stop of thread Console
  try {
    conslrep.stopRunning();
  } catch (Exception ignored) {
  }
  try {
    consl.stopRunning();
  } catch (Exception ignored) {
  }

  // Stop of thread Info
  log("Point E");
  Info.getInstance().stopRunning();
  log("Point F");
  try {
    ata.release();
  } catch (Exception ignored) {
  }
  log("Point G");
  try {
    atb.release();
  } catch (Exception ignored) {
  }
  log("Point H");
  try {
    sleep(1000);
  } catch (InterruptedException ex) {
  }
  log("Point I");
  selectStdout("USB1"); // Switch stdout to safe direction
  try {
    atc.release();
  } catch (Exception ignored) {
  }

  log("Stop completed.");

  machineJumpTo(CTRL_FINISHED);
}

/**
 * ctrlMachineFinished
 */
private void ctrlMachineFinished() {
  bRunning = false;
}

/**
 * delay
 *
 * @param millis long
 */
private void delay(long millis) {
  try {
    sleep(millis);
  } catch (InterruptedException ignored) {
  }
}

// Jump to named state of machine (dtMachineStateH)
private void machineJumpTo(int aNewState) {
  iMachine = aNewState;
}

/**
 * printDeviceInfo()
 *
 * @param sPrefix
 */
public static void printDeviceInfo(Vector src) {

  for (int i = 0; i < src.size(); i++) {
    log((String) src.elementAt(i), Logger.ALWAYS);
  }

}

/**
 * prepareDeviceInfo()
 *
 * @return
 */
public static Vector prepareDeviceInfo() {

  Vector result = new Vector(7);
  result.addElement("--- START --------------------------------------------------------");
  result.addElement("Product:       " + Main.PRODUCT_NAME + " (" + Main.PRODUCT_NUMBER + ")");
  result.addElement("Version:       " + Main.PRODUCT_VERSION);
  result.addElement("Manufacturer:  SEA s.r.o., Czech Republic");
  result.addElement("               www.seapraha.cz");
  result.addElement("Serial Number: " + formatSerialNumber());
  result.addElement("------------------------------------------------------------------");
  return result;
}

/**
 * detectSerialNumber()
 */
private void detectSerialNumber() {

  String sText;
  LineReader lReader;
  String s;
  String sResult = "";

  // Read whole file into a String
  sText = FileUtils.readTextFile(FILENAME_SERIAL_NUMBER);
  lReader = new LineReader(sText);
  // Get the first line
  s = lReader.getLine();
  if (s != null) {
    if (s.length() >= 3) {
      if (s.charAt(0) == 'G') {
        sResult = s.trim();
      }
    }
  }

  // Default serial number for exceptional casea (because SeaConfigurator will get mad without serial number)
  if (sResult.length() == 0) {
    sResult = "GC914001";
  }
  Main.SERIAL_NUMBER = sResult;

}

/**
 * printSerialNumber()
 *
 * @return String
 */
private static String formatSerialNumber() {

  if (Main.SERIAL_NUMBER.length() > 0) {
    return Main.SERIAL_NUMBER;
  } else {
    return "-";
  }

}

/**
 * getStatus
 *
 * @return String
 */
private String getStatus(int _type) {
  String result = "";

  if (_type == 0) {
    result += "A";

  } else {
    result += "  "
      + Main.PRODUCT_NAME + " " + Main.PRODUCT_VERSION + " " + Main.SERIAL_NUMBER;
  }

  return result;
}


/**
 * Set STDOUT to ASC1
 *
 * @param destination "ASC0", "ASC1" or "NULL", "USB1"
 */
public void selectStdout(String destination) {
  try {
    atc.send("at^scfg=\"Userware/Stdout\",\"" + destination + "\"\r");
  } catch (ATCommandFailedException ignored) {
  }
}
}
